use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.002004
eval "use Test::Spelling 0.12; use Pod::Wordlist::hanekomu; 1" or die $@;


add_stopwords(<DATA>);
all_pod_files_spelling_ok('bin', 'lib');
__DATA__
DNS
OpenSSH
ROADMAP
littlechef
runlist
subcommand
subkeys
subshell
thawer
wildcard
webserver
David
Golden
lib
Pantry
App
Model
Command
show
rename
create
init
Util
list
strip
sync
Node
edit
Role
Serializable
apply
Runlist
delete
